/*****************************************************************************
 * vlc_objects.h: vlc_object_t definition.
 *****************************************************************************
 * Copyright (C) 2002 VideoLAN
 * $Id: main.h,v 1.35 2002/04/27 22:11:22 gbazin Exp $
 *
 * Authors: Vincent Seguin <seguin@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#ifdef HAVE_STDLIB_H
#   include <stdlib.h>                                          /* realloc() */
#endif

/* VLC_INIT_OBJECT: initialize a vlc object and set its parent */
#define VLC_INIT_OBJECT( niu, ref ) \
    __vlc_init_object( CAST_TO_VLC_OBJECT(niu), CAST_TO_VLC_OBJECT(ref) )

static inline void __vlc_init_object( vlc_object_t *p_new,
                                      vlc_object_t *p_ref )
{
    memset( p_new, 0, sizeof( vlc_object_t ) );
    p_new->i_pid = 0;
    p_new->psz_type = "unknown";

    p_new->b_die = 0;
    p_new->b_error = 0;
    p_new->b_active = 0;

    p_new->p_this = p_new;
    p_new->p_vlc = p_ref->p_vlc;

    p_new->pp_parents = NULL;
    p_new->i_parents = 0;
    p_new->pp_children = NULL;
    p_new->i_children = 0;
}

/* VLC_ATTACH_PARENT: attach a parent object */
#define VLC_ATTACH_PARENT( thiz, parent ) \
    __vlc_attach_parent( CAST_TO_VLC_OBJECT(thiz), CAST_TO_VLC_OBJECT(parent) )

static inline void __vlc_attach_parent( vlc_object_t *p_this,
                                        vlc_object_t *p_parent )
{
    vlc_mutex_lock( &p_this->p_vlc->structure_lock );

    p_this->i_parents++;
    p_this->pp_parents = (vlc_object_t **)realloc( p_this->pp_parents,
                            p_this->i_parents * sizeof(vlc_object_t *) );
    p_this->pp_parents[p_this->i_parents - 1] = p_parent;

    p_parent->i_children++;
    p_parent->pp_children = (vlc_object_t **)realloc( p_parent->pp_children,
                               p_parent->i_children * sizeof(vlc_object_t *) );
    p_parent->pp_children[p_parent->i_children - 1] = p_this;

    vlc_mutex_unlock( &p_this->p_vlc->structure_lock );
}

/* VLC_ATTACH_CHILD: attach a child object */
#define VLC_ATTACH_CHILD( thiz, child ) \
    __vlc_attach_child( CAST_TO_VLC_OBJECT(thiz), CAST_TO_VLC_OBJECT(child) )

static inline void __vlc_attach_child( vlc_object_t *p_this,
                                       vlc_object_t *p_child )
{
    vlc_mutex_lock( &p_this->p_vlc->structure_lock );

    p_this->i_children++;
    p_this->pp_children = (vlc_object_t **)realloc( p_this->pp_children,
                             p_this->i_children * sizeof(vlc_object_t *) );
    p_this->pp_children[p_this->i_children - 1] = p_child;

    p_child->i_parents++;
    p_child->pp_parents = (vlc_object_t **)realloc( p_child->pp_parents,
                             p_child->i_parents * sizeof(vlc_object_t *) );
    p_child->pp_parents[p_child->i_parents - 1] = p_this;

    vlc_mutex_unlock( &p_this->p_vlc->structure_lock );
}

